﻿using System;
using Scylla.BehaviourTree;
using Scylla.Common;
using Scylla.Common.Gaia;
using Scylla.Common.Math;
using Scylla.CommonBot.Profile;
using Scylla.Navigation;
using Scylla.Wot;
using Action = Scylla.BehaviourTree.Action;

namespace Scylla.CommonBot.Routines
{
	/// <summary>
	/// Default Artillery behaviour.
	/// </summary>
	public class DefaultArtilleryRoutine : ITankRoutine
	{
		#region Implementation of ITankRoutine

		public string Name { get { return "Default Artillery"; } }

		public string Author { get { return "Bossland GmbH"; } }

		public Version Version { get { return new Version(1, 0); } }

		public TankRoutineType Type { get { return TankRoutineType.Artillery; } }

		#endregion

		public Composite CombatBehaviour { get { return CreateCombatBehaviour(); } }

		public Composite RoamBehaviour { get { return CreateRoamBehaviour(); } }

		/// <summary>
		/// Handles any initialisation required to make this TankRoutine function properly..
		/// </summary>
		public void Initialise()
		{
			if (IsInitialised)
				return;

			IsInitialised = true;
			Logging.Write("Artillery is ready comrade.");
		}

		private Hotspot _currentHotspot;
		private Composite CreateRoamBehaviour()
		{
			return new PrioritySelector(
				new Decorator(ret => _currentHotspot == null, CreateSelectHotspotBehaviour()),
				new Decorator(ret => _currentHotspot != null && _currentHotspot.IsValid,
					new Sequence(
						CommonBehaviours.MoveToAndStop(ctx => _currentHotspot, 6f),
						new Action(daCode =>
						{
							Poi.Current = new Poi(PoiType.Roam, "Random hidden camp spot.");
							Logging.WriteDiagnostic("[Artillery] Current Hotspot is: {0}", _currentHotspot);
						})
						)));
		}


		private static DateTime _lastRequest;
		private Composite CreateSelectHotspotBehaviour()
		{


			return new PrioritySelector(
				new Decorator(ret => ServerResults.LastRadiusResult == Vector3.Zero,
					new Action(ret =>
					{
						// Server throttles this too, but there's no point in sending it too often to begin with.
						if (DateTime.Now - _lastRequest <= TimeSpan.FromSeconds(1))
							return RunStatus.Success;

						Navigator.Current.SendHidingSpotRadiusRequest(250f);
						_lastRequest = DateTime.Now;

						return RunStatus.Success;
					})
					),
				new Decorator(ret => ServerResults.LastRadiusResult != Vector3.Zero,
					new Action(ret =>
					{
						var spot = ServerResults.LastRadiusResult;
						Logging.WriteDiagnostic("[Artillery] Selected {0} as the place to camp!", spot);
						_currentHotspot = new Hotspot(spot.X, spot.Y, spot.Z);
					}))
				);
		}

		private Composite CreateCombatBehaviour()
		{
			return CommonBehaviours.CreateKillBehaviour();
		}

		public bool IsInitialised { get; private set; }
		public void Dispose()
		{
			_currentHotspot = null;
		}
	}
}
