﻿using System;
using Scylla.BehaviourTree;
using Scylla.Common;
using Scylla.Common.Math;
using Scylla.Wot;
using Scylla.Wot.Objects;
using Action = Scylla.BehaviourTree.Action;

namespace Scylla.CommonBot.Routines
{
    /// <summary>
    /// The default, basic battle tank routine.
    /// </summary>
    public class DefaultBattleTank : ITankRoutine
    {
        public Composite RoamBehaviour { get { return CreateRoamBehaviour(); } }

        public Composite CombatBehaviour { get { return CreateCombatBehaviour(); } }

        /// <summary>
        /// Handles any initialisation required to make this TankRoutine function properly..
        /// </summary>
        public void Initialise()
        {
            if (IsInitialised)
                return;

            IsInitialised = true;
            Logging.Write("Battle Tank ready for combat!");
        }

        /// <summary>
        /// Creates the roam behaviour.
        /// </summary>
        /// <returns></returns>
        public Composite CreateRoamBehaviour()
        {
            return new PrioritySelector(ctx => ObjectiveManager.FirstObjective, // Objective as context please. Kthx!
                CommonBehaviours.CreateWaypointNavigationBehaviour(),
                // If we're out of waypoints to navigate, just rush the objective from where we are currently.
                new Decorator(ret =>
                    {
                        var objective = ret as WotObjective;
                        if (objective == null)
                            Logging.Write("[{0}] Objective is null, something went terribly wrong!", ScyllaWot.Me.Arena.Name);
                        return objective != null && objective.Location != Vector3.Zero;
                    },
                    new Sequence(
                        CommonBehaviours.MoveToAndStop(ret => (ret as WotObjective).Location.Value, 10f, true),
                        new Action(ret => { Poi.Current = new Poi(PoiType.Objective, "Moving to complete objective", (ret as WotObjective)); })
                    )
                )
            );
        }

        /// <summary>
        /// Creates the combat behaviour.
        /// </summary>
        /// <returns></returns>
        public Composite CreateCombatBehaviour()
        {
            // We'll be using the default kill behaviour for this one. It's basically shoot at stuff that's visible and we have 
            // a clear vision on. We let the game do the heavy lifting to reduce the load on the bot itself.
            return CommonBehaviours.CreateKillBehaviour();
        }

        public bool IsInitialised { get; private set; }

        #region Implementation of ITankRoutine

        public string Name { get { return "Default Battle Tank"; } }
		public string Author { get { return "Bossland GmbH"; } }
        public Version Version { get { return new Version(1, 0); } }
        public TankRoutineType Type { get { return TankRoutineType.BattleTank; } }

        #endregion

        public void Dispose()
        {
            
        }
    }
}
