﻿using System;
using System.Collections.Generic;
using System.Linq;
using Scylla.Common;
using Scylla.CommonBot.Settings;
using Scylla.Wot;
using Scylla.Wot.Objects;

namespace Scylla.CommonBot.TargetingProviders
{
    class DefaultArtilleryTargeting : ITargetingProvider
    {
        private CachedValue<WotObject> _target = new CachedValue<WotObject>(() => InternalGetObjectsByWeight().FirstOrDefault(), TimeSpan.FromSeconds(10)); 

        public WotObject FirstObject { get { return _target.Value; } }

        public WotVehicle FirstVehicle { get { return FirstObject as WotVehicle; } }

        public List<WotObject> GetObjectsByWeight()
        {
            return InternalGetObjectsByWeight();
        }

        internal static List<WotObject> InternalGetObjectsByWeight()
        {
            if (ObjectManager.VisibleObjects == null)
                return new List<WotObject>();

            var list = ObjectManager.GetObjects<WotVehicle>().Where(IsValid).Select(s => new Score()
                                                                                             {
                                                                                                 Object = s,


                                                                                                 Weight = 100 + (1000 - (s.Speed * 10)) + (s.Tier * 10) + s.Distance
                                                                                             });

            return list.OrderByDescending(v => v.Weight).Select(veh => veh.Object).ToList();
        }

        public TargetingType Type { get { return TargetingType.Artillery; } }

        public string Name { get { return "Default Artillery Targeting"; } }
		public string Author { get { return "Bossland GmbH"; } }

        private static bool IsValid(WotObject obj)
        {
            var o = obj as WotVehicle;

            if (o == null)
                return false;

            if (!o.IsAlive)
                return false;

            // We don't want to be shooting at moving guys for artillery.
            if (o.Speed > 10 || (GlobalSettings.Instance.IgnoreMovingTargets && o.Speed > 0))
                return false;

            // Don't be shootin' at no friendlies.
            if (o.IsFriendly)
                return false;

            return true;
        }

        public void Dispose()
        {
        }
    }
}
