﻿using System;
using System.Collections.Generic;
using System.Linq;
using Scylla.Common;
using Scylla.CommonBot.Settings;
using Scylla.Wot;
using Scylla.Wot.Objects;

namespace Scylla.CommonBot.TargetingProviders
{
    /// <summary>
    /// Default TankLeader targeting provider, which works for battle tanks and tank destroyers.
    /// </summary>
    public class DefaultTargeting : ITargetingProvider
    {
        private CachedValue<WotObject> _target = new CachedValue<WotObject>(() => InternalGetObjectsByWeight().FirstOrDefault(), TimeSpan.FromSeconds(3)); 

        public WotObject FirstObject { get { return _target.Value; } }

        public WotVehicle FirstVehicle { get { return FirstObject as WotVehicle; } }

        public List<WotObject> GetObjectsByWeight()
        {
            return InternalGetObjectsByWeight();
        }

        internal static List<WotObject> InternalGetObjectsByWeight()
        {
            if (ObjectManager.VisibleObjects == null)
                return new List<WotObject>();

            try
            {
                var weighedTargets = ObjectManager.VisibleObjects.Where(InternalIsValid).Select(s =>{
                    var score = new Score
                    {
                        Object = s,

                        // We want to start off with a beefy amount of weight, 
                        // so we can gradually steal a bit of it everytime. Muhahahahhaehuheuheuheu!
                        Weight = 5000 - (10 * s.Distance) - (10 * (100 - s.HealthPercentage)) - (s.Tier * 50) + (s.CanHit ? 1000 : 5)
                    };
                    return score;
                });


                return weighedTargets.OrderByDescending(s => s.Weight).Select(o => o.Object).ToList();
            }
            catch (Exception)
            {
                // We don't really care about exceptions. It is possible that the ObjectManager updates in between target
                // selections, which will throw a null reference exception and cause the whole thing to go south, but all we care about
                // is that when that happens, we have no targets to focus.
                return new List<WotObject>();
            }
        }

        private bool IsValid(WotVehicle obj)
        {
            return InternalIsValid(obj);
        }

        private static bool InternalIsValid(WotVehicle obj)
        {
            if (obj == null)
                return false;

            if (!obj.IsAlive)
                return false;

            if (GlobalSettings.Instance.IgnoreMovingTargets && obj.Speed > 0)
                return false;

            if (obj.IsFriendly)
                return false;

            if (!obj.CanHit)
                return false;

            return true;
        }

        public TargetingType Type { get { return TargetingType.BattleTank | TargetingType.TankDestroyer; } }

        public string Name
        {
            get { return "Default Targeting"; }
        }

		public string Author { get { return "Bossland GmbH"; } }

        public void Dispose()
        {
        }
    }
}