﻿using System;
using System.Collections.Generic;
using System.Linq;
using Scylla.Common;
using Scylla.Wot;
using Scylla.Wot.Objects;

namespace Scylla.CommonBot.TargetingProviders.DefaultTargetingProviders
{
    public class ExecuteTargetProvider : ITargetingProvider
    {
        private CachedValue<WotObject> _target = new CachedValue<WotObject>(() => InternalGetObjectsByWeight().FirstOrDefault(), TimeSpan.FromSeconds(3)); 

        public void Dispose()
        {
        }

        public WotObject FirstObject { get { return _target.Value; } }

        public List<WotObject> GetObjectsByWeight()
        {
            return InternalGetObjectsByWeight();
        }

        internal static List<WotObject> InternalGetObjectsByWeight()
        {
            var targets = ObjectManager.VisibleObjects.Where(IsValid).Select(s => new Score
            {
                Object = s,

                Weight = 100 - s.HealthPercentage
            });

            return targets.OrderByDescending(t => t.Weight).Select(t => t.Object).ToList();
        }

        private static bool IsValid(WotVehicle obj)
        {
            if (!obj.IsAlive)
                return false;

            if (obj.IsFriendly)
                return false;

            if (!obj.CanHit)
                return false;

            return true;
        }

        public TargetingType Type { get { return TargetingType.Other; } }
        public string Name { get { return "Excecute Targeting"; } }
		public string Author { get { return "Bossland GmbH"; } }
    }
}
